import config as cf
from model_initialize import initialize_f, param_f, var_f
from model_constr import constr_f
from model_result import result_f
from model_veh_result import cars_leave_f, store_route_f

from gurobipy import *
import numpy as np
 
def cc_model_f(test, s_dict, r_dict, G_o, p_nodes, arrivals, total_exp_demand, 
               scenarios, origins, n_ins):
    if test == 'in':
        print('Chance Constrained Model -- In Sample Test')
    
    S, R, V, cars, y_ind = initialize_f(s_dict, r_dict, G_o, scenarios)
    M = total_exp_demand*cf.M_multiplier
    
    Z = param_f(test, 'Chance Constrained Model')
    
    (b, d, x, y, z, q, mu, k) = var_f(Z, S, R, V, y_ind, scenarios, p_nodes, 
                                      test)
    
    if test == 'in':
        Z.setObjective(quicksum(x[s] for s in S), GRB.MINIMIZE)
        
    elif type(test) == int:
        Z.setObjective(-quicksum(mu[sc] for sc in scenarios), GRB.MINIMIZE)

    constr_f(Z, b, d, x, y, z, q, mu,
             S, R, V, s_dict, r_dict,
             cars, G_o, p_nodes, arrivals, scenarios, test)
    
    Z.addConstrs(quicksum(d[v,sc] for v in V)<=cf.D for sc in scenarios)
    
    if test == 'in':
        cc_in_constr_f(Z, mu, M, k, arrivals, scenarios)

    Z.optimize()
    (obj, rt, obj_sc, opt_gap, evac, 
     percent_evac, total_dist_traveled, 
     budget, loc_evac, loc_demand) = result_f(Z, mu, d, b, q, V, scenarios, 
                                              arrivals, p_nodes, n_ins, test)

    if test == 'in':
        s_hired = {s for s in S if x[s].x == 1}
        k_total = np.array([k[sc].x for sc in scenarios]).sum()
    elif type(test) == int:
        s_hired = S
        k_total = None
    
    r_hired = [{r for r in R if z[r,sc].x == 1} for sc in scenarios]
    s_hired_dict = {s: s_dict[s] for s in s_hired} 
    
    s_leave = cars_leave_f(s_hired, scenarios, scenarios, G_o, cars, y)
    r_leave = [cars_leave_f(r_hired[sc], [sc], scenarios, G_o, cars, y)[sc]
               for sc in scenarios]
    
    num_r_cars = np.array([float(len(r_hired[sc])) for sc in scenarios])
    s_hired_dict = {s: s_dict[s] for s in s_hired}
    
    r_hired_dict = [{r: r_dict[r] for r in r_hired[sc]} for sc in scenarios]
        
    if type(test) == int and cf.store_vehicle_routing_solns:
        veh_routing_dict = {'percent evacuated': percent_evac[0], 
                            'unhired r cars':[]}
        for s in s_hired:
            veh_routing_dict[s] = store_route_f(G_o, cars, s, y, q)   
            veh_routing_dict[s].update(s_dict[s])
        for r in R:
            if r in r_hired[0]:
                veh_routing_dict[r] = store_route_f(G_o, cars, r, y, q)   
                veh_routing_dict[r].update(r_dict[r])
            else:
                veh_routing_dict['unhired r cars'].append(r)
        
    else: 
        veh_routing_dict = None
      
    return (k_total,
            {'obj': float(obj),
            'time': float(rt),
            'dist': total_dist_traveled,
            's_leave': s_leave,
            'r_leave': r_leave,
            'r_cars': num_r_cars,
            'percent_evac': percent_evac,
            'evac': evac[:,0],
            't_demand': evac[:,1]},
            float(opt_gap),
            obj_sc,
            budget,
            s_hired_dict,
            r_hired_dict,
            veh_routing_dict,
            loc_demand,
            loc_evac
            )
            
def cc_in_constr_f(Z, mu, M, k, arrivals, scenarios):
    
    '''
    These constraints ensure that candidate solution x allows gamma proportion
    of evacuees to be evacuated in more than 1-epsilon proportion of scenarios.
    '''
    
    Z.addConstrs(mu[sc] + M*k[sc] >= cf.gamma*float(np.array(arrivals[sc]).sum()) 
                 for sc in scenarios)
    
    Z.addConstr(quicksum(k[sc] 
                         for sc in scenarios
                         ) <= np.floor(cf.epsilon*float(len(scenarios)))
                )
    
