'''
Expectation-Based and Chance-Constrained Model Options
'''
model_options = ['exp', 'cc', 'heuristic']
model_type = model_options[2]

os_test_options = ['exp','cc']
os_test_type = os_test_options[0]

if model_type == 'exp' or os_test_type == 'exp':
    wt=[.999,.001] #weights of the objectives

if model_type == 'cc' or os_test_type == 'cc':
    # probabilistic constraint parameters
    gamma = 1 # desired proportion of evacuees served 
    epsilon = .05 # error threshold
    
    D = 230 # total distance traveled for all cars upper bound parameter
    
    M_multiplier = 3 # multiply this value by total expected demand  
    # to get big M constant in integer LP formulation of chance constraint
    # increase M_multiplier for demand distributions with larger variance
    

'''
In Sample Testing Options
'''

findCandidateSoln = True 
# if True, perform in-sample test to find candidate solns out of 
# fleet of vehicles available,
# and then perform out-of-sample test on candidate solns; 
# if False, hire entire fleet of vehicles available
# and then perform out-of-sample test for the fleet 

if findCandidateSoln:
    nsample=5
    nscenario=100

else:
    nsample=1
              
'''
Out of Sample Testing Options
'''
performOutofSampleTest = True

if performOutofSampleTest:
    noutofsample = 1000
    store_vehicle_routing_solns = False

else:
    noutofsample = 1
    store_vehicle_routing_solns = False

'''
Computational Settings
'''
if findCandidateSoln and model_type != 'heuristic':
    in_MIP_gap = .01
    in_model_Relax = False #if True, relax y,z,q for the in sample test
    
os_MIP_gap = .01
timeLimit = 3600

'''
Evacuee Demand Scenario Parameters
'''
scen_data = 'read'
demand_mean_file = 'data/charleston_demand_28_periods.csv'
# include this even if we are reading scenario data, to calculate
# expected demand

if scen_data == 'generate':
    in_dist='normal'  
    os_dist='normal'
    demand_variance_proportion = .3
    store_scen_data = True

elif scen_data == 'read':
    r_folder='Omega/'
    store_scen_data = False

'''
Network Parameters
'''
# distance (m) and time (s) between all nodes in a spatial supernetwork -
# Charleston County
network_dist_file = 'data/charleston_network_distance_all.csv'
network_time_file = 'data/charleston_network_duration_all.csv'

# distance (m) and time (s) between all nodes and highway
# in spatial supernetwork
h_time = {i: 1 for i in range(48)}
h_dist = {i: 10 for i in range(48)}

# 5 spatial regional subnetworks: 
# names, pickup zones in that region, origin zones 
# for volunteers and emergency vehicles in that region  
n_names = ['Region A','Region B','Region C', 'Region D', 'Region E']
pickup = [[12,33,37,38], 
          [2,4,20,21,24,26,32,43,44,46],
          [0,9,13,14,17,18,22,23,45],
          [5,31,25,7,34,41,8,47,16,39,40,6,1,11,35,15],
          [3,27,28,30,29,42,10,19,36]]
s_origins = [reg_pickup_zones for reg_pickup_zones in pickup]
r_origins = [[12],[24],[45],[41],[27]] 

# type of spatiotemporal network to construct
networkType = '1_h_1_o'# 1 h node at T+1 for safe zone, 
                       # 1 Ov node at 0 for each origin zone

distOrigin_to_Pickup_same_zone = 0 # presumed distance traveled
                                   # when a vehicle stays in its origin zone
                                   # for pickup

# indices of regional subnetworks to test this run
networks = [4]

'''
Vehicle Spatial Parameters

Note: For Option 0 and 1, only one network and hour can be examined
in a single program run.
For Options 2 and 3, multiple networks and hours can be examined in a 
single program run. 
'''

car_spatial_dist_options = ['fix total and distribution',
                              'calculate total cars and distribution',
                              'fix total, calculate distribution',
                              'read total and distribution']

car_spatial_dist = car_spatial_dist_options[3]

if car_spatial_dist == 'fix total and distribution':
    s_cars_per_node = {5:1,31:2,25:1,7:0,34:0,41:0,8:0,47:4,16:4,39:1,40:2,
                       6:1,1:2,11:2,35:1,15:2}
    r_cars_per_node = {5:0,31:0,25:0,7:0,34:0,41:6,8:0,47:0,16:0,39:0,40:0,
                       6:0,1:0,11:0,35:0,15:0}
elif car_spatial_dist == 'read total and distribution':
    s_vehicle_dist_file = 's_vehicle_distribution.csv'
    r_vehicle_dist_file = 'r_vehicle_distribution.csv'
    
    if scen_data == 'generate':
        r_folder = 'Omega2/' # otherwise, you read from the same folder as the
                            # scenario data
    
else:
    s_cars_spatial_dist_file = 'data/charleston_volunteers_per_node.csv'
    r_cars_spatial_dist_file = 'data/charleston_emergency_per_node.csv'
    
    if car_spatial_dist == 'calculate total cars and distribution':
        vehicledistribution = (.6,.4) # (s planned cars, r emergency cars)
        excess_cars_available = 2 # have about 2 times as many as exp demand
    
    elif car_spatial_dist == 'fix total, calculate distribution':
        total_s_cars = 8
        total_r_cars = 7

 
    
'''
Vehicle Cost and Capacity Parameters
(cost unit: 10 dollars)
'''

random_s_parameters = False
random_r_parameters = False

if random_s_parameters:
    s_param = [(3,1), (4,2), (5,3), (6,4)]
    s_p = [.2,.3,.3,.2]
    expected_s_capacity = sum(s_param[i][1]*s_p[i] 
                              for i in range(len(s_param)))
    max_s_capacity = max(s_param[i][1] for i in range(len(s_param)))
else:
    s_cost = 5 
    s_capacity = 3
    expected_s_capacity = s_capacity
    max_s_capacity = s_capacity

if random_r_parameters:
    r_param = [(10,5), (15,10)]
    r_p = [.75,.25]
    expected_r_capacity = sum(r_param[i][1]*r_p[i] 
                              for i in range(len(r_param)))
    max_r_capacity = max(r_param[i][1] for i in range(len(r_param)))
else:
    r_cost = 10
    r_capacity = 7
    expected_r_capacity = r_capacity
    max_r_capacity = r_capacity
maxQ = max(max_s_capacity,max_r_capacity) + 1

B = 2000 # total budget

'''
Time Parameters
'''
T = 4 # number of intervals in a period
hours = [6] # list of hours to test
interval = 900 #length of each interval t, in seconds


'''
Options for Storage of Results
'''
w_folder='heuristic_soln_E_7/'
