import config as cf
from data_generation import sample_scen_f
from make_folder import make_folder_f
from read_data import read_scen_f
from exp_model import exp_model_f
from cc_model import cc_model_f
from heuristic import heuristic_f
from store_results import store_results_f
from data_overview_columns import in_col_names_f

import pandas as pd
import numpy as np

def find_candidate_f(s_dict, r_dict, s_cars_at_node, G_o, p_nodes, 
                     origins, h_demand_mean, exp_n_h_demand, t_final,  
                     distance, n_ins, sample, hour, n_data_overview,  
                     w_n_folder, w_h_folder, scen_folder):           
 
    if cf.scen_data == 'generate':
        in_scen = [sample_scen_f(n_ins, cf.in_dist, h_demand_mean, t_final) 
                   for sc in range(cf.nscenario)]
        
        if cf.store_scen_data == True:
            w_in_folder = make_folder_f(scen_folder, 'Sample '+str(sample+1))
                    
            for sc in range(cf.nscenario):
                scen_df = None
                scen_df = pd.DataFrame(in_scen[sc])
                scen_df.to_excel(''.join([w_in_folder,'Scenario ',str(sc+1),
                                          '.xlsx'])) 
    
    elif cf.scen_data == 'read':
        r_in_folder = make_folder_f(scen_folder, 'Sample '+str(sample+1))
        in_scen = [read_scen_f(r_in_folder,sc) for sc in range(cf.nscenario)]
                
    if cf.model_type == 'exp':
        (result_in, opt_gap, 
         obj_sc, budget, 
         s_hired_dict,
         r_hired_dict,
         veh_routing_dict, 
         loc_demand, loc_evac) = exp_model_f('in', s_dict, r_dict, G_o, 
                                             p_nodes, in_scen, 
                                             range(cf.nscenario), origins, 
                                             n_ins)                                        
        k = None
    elif cf.model_type == 'cc':
        (k, result_in, opt_gap, 
         obj_sc, budget, 
         s_hired_dict,
         r_hired_dict,
         veh_routing_dict, 
         loc_demand, loc_evac) = cc_model_f('in', s_dict, r_dict, G_o, 
                                            p_nodes, in_scen, exp_n_h_demand, 
                                            range(cf.nscenario), origins, 
                                            n_ins)
        
    if cf.model_type == 'exp' or cf.model_type == 'cc':
        store_results_f(result_in, opt_gap, k, obj_sc, budget,
                           w_h_folder, sample, range(cf.nscenario))
 
        n_data_overview[n_ins][0].append([cf.n_names[n_ins], 
                                      hour+1, sample+1,
                                      result_in['time'], opt_gap,
                                      k, cf.nscenario, result_in['obj'], 
                                      np.mean(result_in['percent_evac']),
                                      np.var(result_in['percent_evac']),
                                      np.min(result_in['percent_evac']),
                                      exp_n_h_demand,
                                      np.mean(result_in['t_demand']),
                                      np.var(result_in['t_demand']),
                                      np.mean(result_in['dist']),
                                      np.var(result_in['dist']),
                                      len(s_hired_dict.keys()),
                                      len(s_dict.keys()),
                                      np.mean(result_in['s_leave']),
                                      np.var(result_in['s_leave']),
                                      np.mean(result_in['r_cars']),
                                      len(r_dict.keys()),
                                      np.var(result_in['r_cars']),
                                      np.mean(result_in['r_leave']),
                                      np.var(result_in['r_leave']),
                                      s_hired_dict])
                    
        datadf = None
        datadf = pd.DataFrame(n_data_overview[n_ins][0], columns=in_col_names_f())
        datadf.to_excel(''.join([w_n_folder,'In Sample Overview.xlsx']))
    
    elif cf.model_type == 'heuristic':
        s_hired_dict, rt = heuristic_f(s_dict, r_dict, in_scen,
                                       s_cars_at_node, distance, n_ins)
        
        result_in = {'time': rt}
        
        result_categories = ['obj','dist','s_leave','r_leave','r_cars',
                             'percent_evac','evac','t_demand']
        
        for category in result_categories:
            result_in[category] = None
        
        store_results_f(result_in, 
                        None, None, None, None, 
                        w_h_folder, sample, range(cf.nscenario))
        
        n_data_overview[n_ins][0].append([cf.n_names[n_ins], 
                                      hour+1, sample+1,
                                      result_in['time'], cf.nscenario, 
                                      exp_n_h_demand,
                                      len(s_hired_dict.keys()),
                                      len(s_dict.keys()),
                                      len(r_dict.keys()),
                                      s_hired_dict])
        
        datadf = None
        datadf = pd.DataFrame(n_data_overview[n_ins][0],columns=['Region',
                                                                 'Hour',
                                                                 'Sample',
                                                                 'Runtime',
                                                                 '# Scen',
                                                                 'Tot Exp Dem',
                                                                 'Hired S',
                                                                 'S Avail',
                                                                 'R Avail',
                                                                 'S Hired Cars'
                                                                 ])
        datadf.to_excel(''.join([w_n_folder,'In Sample Overview.xlsx']))
        
    return n_data_overview, s_hired_dict