import config as cf

from gurobipy import *

def constr_f(Z, 
             b, d, x, y, z, q, mu,
             S, R, V, 
             s_dict, r_dict, cars,
             G_o, p_nodes,
             arrivals, scenarios, test):    

    # in out of sample test, all volunteer vehicles have been hired
    if type(test) == int:
        Z.addConstrs(x[s] == 1
                     for s in S)
        
    # volunteer vehicle s in S leaves its origin if it is hired (x[s] == 1)       
    Z.addConstrs(quicksum(y[s,it,jtt,sc] 
                          for it, jtt in G_o[
                                      s_dict[s]['origin']
                                      ].out_edges(
                                          (s_dict[s]['origin'],0)
                                          )
                          ) == x[s] 
                for s in S 
                for sc in scenarios) 
                                          
    # emergency vehicle r in R leaves its origin 
    # if it is hired in sc (z[r,sc] == 1)
    Z.addConstrs(quicksum(y[r,it,jtt,sc] 
                          for it, jtt in G_o[
                                  r_dict[r]['origin']
                                  ].out_edges(
                                      (r_dict[r]['origin'],0)
                                      )
                          ) == z[r,sc] 
                 for r in R 
                 for sc in scenarios) 
                                      
    # flow constraints for v in V for all nodes in the subgraph 
    # reachable from Ov
    Z.addConstrs(quicksum(y[v,it,jtt,sc] 
                          for it, jtt in G_o[
                                  cars[v]['origin']
                                  ].out_edges(n)
                          )
                 -quicksum(y[v,jtt,it,sc] 
                           for jtt, it in G_o[
                                   cars[v]['origin']
                                   ].in_edges(n)
                           )
                        == 0 
                for v in V 
                for n in G_o[
                        cars[v]['origin']
                        ].nodes 
                for sc in scenarios)                          
    
    # vehicle v in V is only loaded at pickup node (i,t) if v visits (i,t)
    Z.addConstrs(q[v,i,t,sc] <= cf.maxQ*quicksum(y[v,jtt,it,sc] 
                                                 for jtt, it in G_o[
                                                         cars[v]['origin']
                                                         ].in_edges((i,t)
                                                                    )
                        ) 
                for v in V
                for i, t in p_nodes 
                for sc in scenarios) 
    
    # capacity constraint for vehicle v in V
    Z.addConstrs(quicksum(q[v,i,t,sc] 
                          for i, t in p_nodes
                          ) 
                 <= cars[v]['capacity'] 
                 for v in V 
                 for sc in scenarios)
    
    # all evacuees picked up at (i,tt) cannot exceed those who have arrived
    # minus those who have already been picked up
    Z.addConstrs(quicksum(q[v,i,tt,sc] 
                          for v in V
                          )
                 <= quicksum(arrivals[sc][i][t] 
                             for t in range(tt+1)
                             )
                 -quicksum(q[v,i,t,sc] 
                           for v in V 
                           for t in range(tt) 
                           if (i,t) in p_nodes
                           ) 
                 for sc in scenarios 
                 for i, tt in p_nodes)
    
    # create a variable to track number of evacuees served
    Z.addConstrs(quicksum(q[v,i,t,sc] 
                          for v in V 
                          for i, t in p_nodes
                          ) == mu[sc]
                 for sc in scenarios)
    
    # create a variable to track budget
    Z.addConstrs(quicksum(x[s]*s_dict[s]['cost'] 
                          for s in S
                          )
                +quicksum(z[r,sc]*r_dict[r]['cost'] 
                          for r in R
                          ) == b[sc]
                for sc in scenarios)
   
    # budget constraint
    Z.addConstrs(b[sc] <= cf.B  
                 for sc in scenarios)
    
    # create a variable to track total distance traveled by all v in V
    Z.addConstrs(quicksum(y[v,it,jtt,sc]*dist 
                          for it, jtt, dist in G_o[
                                  cars[v]['origin']
                                  ].edges(data='dist')
                          ) == d[v,sc]
                 for v in V 
                 for sc in scenarios)