import config as cf

import csv
import os

def store_parameters_f(filename=''.join([cf.w_folder,
                                         'Instance_Parameters.csv'])):
    if not os.path.isdir(cf.w_folder):
        os.mkdir(cf.w_folder)
        
    with open(filename, 'w') as csvfile:
        writer = csv.writer(csvfile, dialect='excel')
        
        if cf.model_type == 'exp':
            writer.writerow(['Model type','Expectation-Based Model'])
            writer.writerow(['Objective weight of evacuees served',cf.wt[0]])
            writer.writerow(['Objective weight of distance traveled',cf.wt[1]])
            
        elif cf.model_type == 'cc': 
            writer.writerow(['Model type','Chance-Constrained Model'])
            writer.writerow(['Gamma (proportion of demand served)',cf.gamma])
            writer.writerow(['Epsilon (probabilistic threshold 1-epsilon)', 
                             cf.epsilon])
            writer.writerow(['Total distance traveled upper bound', cf.D])
        else:
            writer.writerow(['Algorithm type','Heuristic'])
            
        if cf.findCandidateSoln:
            writer.writerow(['Number of in sample scenarios',cf.nscenario])
            
            if cf.model_type != 'heuristic':
                writer.writerow(['In sample model relax?', cf.in_model_Relax])
                writer.writerow(['In sample MIP gap', cf.in_MIP_gap])
        
        if cf.performOutofSampleTest:
            writer.writerow(['Number of out sample scenarios',cf.noutofsample])
            writer.writerow(['Out of sample MIP gap', cf.os_MIP_gap])
 
        writer.writerow(['Time limit, all tests (sec)', cf.timeLimit])
        
        if cf.scen_data == 'generate':
            writer.writerow(['Scenarios generated this run using demand means '
                             +'file.'])
            writer.writerow(['In sample distribution',cf.in_dist])
            writer.writerow(['Out of sample distribution',cf.os_dist])
            writer.writerow(['Demand variance- proportion of mean:', 
                             cf.demand_variance_proportion])
        
        elif cf.scen_data == 'read':
            writer.writerow(['Scenarios read from folder',cf.r_folder])
        
        writer.writerow(['Total expected demand for a network and hour '
                         +'calculated with demand means from file',
                         cf.demand_mean_file])
        writer.writerow(['Spatial supernetwork travel distance matrix file', 
                         cf.network_dist_file])
        writer.writerow(['Spatial supernetwork travel time matrix file', 
                         cf.network_time_file])
        writer.writerow(['Travel distance from nodes in supernetwork to '
                         +'highway safe zone',cf.h_dist])
        writer.writerow(['Travel time from nodes in supernetwork to highway'
                         +' safe zone',cf.h_time])
        writer.writerow(['Spatial subnetwork details'])
        writer.writerow(['Network','# Pickup Zones','# Planned Origin Nodes',
                         '# Emergency Origin Nodes',
                     'Pickup Nodes Chosen'])
        for n_ins in cf.networks:
            writer.writerow([cf.n_names[n_ins],len(cf.pickup[n_ins]),
                             len(cf.s_origins[n_ins]),
                             len(cf.r_origins[n_ins]),
                             cf.pickup[n_ins]])
        
        writer.writerow([])
        writer.writerow(['Type of subnetwork',cf.networkType])
        writer.writerow(['Assumed distance time from volunteer origin node '
                         +'to pickup node if same physical zone',
                         cf.distOrigin_to_Pickup_same_zone])
        
        if cf.car_spatial_dist == 'fix total and distribution':
            for s_orig in cf.s_origins[cf.networks[0]]:
                writer.writerow([''.join(['Os',str(s_orig),' cars']),
                                 cf.s_cars_per_node[s_orig]])
            writer.writerow(['Or cars',cf.r_cars_per_node])
        
        elif cf.car_spatial_dist == 'read total and distribution':
            writer.writerow(['Read car distribution from file.'])
            writer.writerow(['Folder',cf.r_folder])
            writer.writerow(['r file name',cf.r_vehicle_dist_file])
            writer.writerow(['s file name',cf.s_vehicle_dist_file])
            
        else:
            writer.writerow(['Planned cars spatial distribution file',
                             cf.s_cars_spatial_dist_file])
            
            if cf.car_spatial_dist == 'fix total, calculate distribution':
                writer.writerow(['Planned cars number',cf.total_s_cars])
                writer.writerow(['Emergency cars number',cf.total_r_cars])
            
            else:
                writer.writerow(['Planned cars proportion',
                                 cf.vehicledistribution[0]])
                writer.writerow(['Emergency cars proportion',
                                 cf.vehicledistribution[1]])
                writer.writerow(['Proportion Vehicles available/'
                                 +'those that can handle demand',
                                 cf.excess_cars_available])
        
        if cf.random_s_parameters:
            writer.writerow(['Planned car (cost in $10 units, capacity)',
                             cf.s_param])
            writer.writerow(['Probabilities of each (cost, capacity) pair',
                             cf.s_p])
        
        else:
            writer.writerow(['Planned car cost',cf.s_cost])
            writer.writerow(['Planned car capacity',cf.s_capacity])
        
        if cf.random_r_parameters:
            writer.writerow(['Emergency car (cost in $10 units, capacity)',
                             cf.r_param])
            writer.writerow(['Probabilities of each (cost, capacity) pair',
                             cf.r_p])
        
        else:
            writer.writerow(['Emergency car cost',cf.r_cost])
            writer.writerow(['Emergency car capacity',cf.r_capacity])
        
        writer.writerow(['Budget',cf.B])
        writer.writerow(['T',cf.T])
        writer.writerow(['Hours tested',cf.hours])
        writer.writerow(['Length of intervals t',cf.interval])

def store_vehicle_parameters_f(s_cars_at_node, r_cars_at_node, folder):
    filename = ''.join([folder,'Vehicle_Parameters.csv'])
    
    with open(filename, 'w') as csvfile:
        writer = csv.writer(csvfile, dialect='excel')
        writer.writerow(['Origin','Car','Capacity','Cost'])
        
        for r_orig in r_cars_at_node.keys():
            
            for r_car in r_cars_at_node[r_orig].keys():
                writer.writerow([r_orig,r_car,
                                 r_cars_at_node[r_orig][r_car]['capacity'],
                                 r_cars_at_node[r_orig][r_car]['cost']])
         
        for s_orig in s_cars_at_node.keys():
            
            for s_car in s_cars_at_node[s_orig].keys():
                writer.writerow([s_orig,s_car,
                                 s_cars_at_node[s_orig][s_car]['capacity'],
                                 s_cars_at_node[s_orig][s_car]['cost']])
            
def store_results_f(result, opt_gap, k, obj_sc, budget,
                       folder, sample, scenarios):
    if sample == 'os':
        file_name = ''.join([folder,'Out of Sample.csv'])
    
    else:
        file_name = ''.join([folder,'Sample ',str(sample+1),'.csv'])

    with open(file_name,'w') as csvfile:
        writer = csv.writer(csvfile)
        writer.writerow(['Objective',result['obj']])
        writer.writerow(['Runtime',result['time']])
        
        if cf.model_type != 'heuristic':
            writer.writerow(['Opt Gap',opt_gap])
            writer.writerow(['Scenario','k','Obj','Total Demand','Total Evac',
                             '% Evac','Emergency Cars','Dist','Budget',
                             'Planned Leave','Emer Leave'])
        
            for sc in scenarios:
                writer.writerow([sc+1, k, obj_sc[sc], result['t_demand'][sc], 
                                 result['evac'][sc], 
                                 result['percent_evac'][sc], 
                                 result['r_cars'][sc], result['dist'][sc],
                                 budget[sc], result['s_leave'][sc], 
                                 result['r_leave'][sc]]) 